const FLD_PROP = "Property";
const FLD_SPROP = "SubProperty";
const FLD_VAL = "Value";

const PRP_PA = "PhysAttr";
const PRP_RD = "RotDoor";
const PRP_TD = "TransDoor";

const MAX_SUBPROP = 4;

class SetVectorProp extends SqRootScript {

	// Get the specified property.
	function GetPropParam() {
		try { return userparams()[GetClassName() + FLD_PROP].tostring(); }
		catch (err)
		{
			Debug.MPrint(
				"[" + GetClassName() + "] No property specified for object "
				+ self + "."
			);
			return null;
		}
	}

	// Get the specified subproperties.
	function GetSubPropParam() {
		local subprops = [];
		subprops.resize(MAX_SUBPROP);
		for (local i = 0; i < MAX_SUBPROP; i++)
		{
			try
			{
				subprops[i] = userparams()[GetClassName() + FLD_SPROP
					+ (0 == i ? "" : i + 1)].tostring();
			}
			catch (err) { }
		}
		return subprops;
	}

	// Get the specified property values.
	function GetValueParam() {
		local values = [];
		values.resize(MAX_SUBPROP);
		for (local i = 0; i < MAX_SUBPROP; i++)
		{
			try {
				values[i] = userparams()[GetClassName() + FLD_VAL
					+ (0 == i ? "" : i + 1)].tostring();
			} catch (err) { }
		}
		return values;
	}

	// Set the requested property fields with the provided values.
	function SetProp() {
		// Fetch parameters for the property to be modified, the subproperties
		// (if applicable), and the vector values to be set.
		local prop = GetPropParam();
		if (null == prop)
			return;
		local subprops = GetSubPropParam();
		local vals = GetValueParam();
		for (local i = 0; i < MAX_SUBPROP; i++)
		{
			// Skip this field if no value was specified.
			if (null == vals[i])
				continue;
			// Separate the comma-separated values passed as the value
			// parameter.
			local val = split(vals[i], ",");
			// Ensure a 3-tuple was passed as the value.
			// If one was, construct a vector using it and populate the
			// specified property field.
			if (3 == val.len()) {
				local vec = vector(val[0].tofloat(), val[1].tofloat(),
					val[2].tofloat());
				Property.Set(self, prop,
					null == subprops[i] ? "" : subprops[i], vec);
			}
		}
		// Handle special cases for modifying physics attributes of doors.
		// Ex. COG Offset
		// In particular, re-initialize the relevant door property.
		if (prop == PRP_PA)
		{
			if (Property.Possessed(self, PRP_RD))
				Property.CopyFrom(self, PRP_RD, self);
			else if (Property.Possessed(self, PRP_TD))
				Property.CopyFrom(self, PRP_TD, self);
		}
	}

	// Set the property on simulation start.
	function OnSim() {
		if (message().starting) SetProp();
	}

}

class SetVectorProp2 extends SetVectorProp { }

class SetVectorProp3 extends SetVectorProp { }

class SetVectorProp4 extends SetVectorProp { }
